{% extends "!layout.html" %}
  {% block menu %}
    <p class="link caption"><span class="link caption-text">Documentation</span></p>
    {{ super() }}
    <br />
    <p class="link caption"><span class="link caption-text">AnaFlow Links</span></p>
    <a href="https://github.com/GeoStat-Framework/AnaFlow">AnaFlow GitHub</a>
    <a href="https://doi.org/10.5281/zenodo.1135723">AnaFlow Zenodo DOI</a>
    <a href="https://pypi.org/project/anaflow/">AnaFlow PyPI</a>
    <br />
    <p class="link caption"><span class="link caption-text">GeoStat Framework</span></p>
    <a href="https://geostat-framework.org">GeoStat Website</a>
    <a href="https://github.com/GeoStat-Framework">GeoStat Github</a>
    <a href="https://geostat-framework.readthedocs.io">GeoStat ReadTheDocs</a>
    <a href="https://pypi.org/user/geostatframework/">GeoStat PyPI</a>
    <br />
    <br />
    <a href="genindex.html">Index</a>
    <a href="contents.html">Sitemap</a>
  {% endblock %}
